# Copyright (C) 2021 OpenAnolis Community
#
#
import re
import os
import copy
from time import sleep
from pyanaconda.flags import flags
from pyanaconda.ui.categories.software import SoftwareCategory
from pyanaconda.ui.tui.spokes import NormalTUISpoke
from pyanaconda.threading import threadMgr, AnacondaThread
from pyanaconda.payload.manager import payloadMgr, PayloadState
from pyanaconda.payload.errors import DependencyError, NoSuchGroup
from pyanaconda.core.i18n import N_, _, C_

from pyanaconda.core.constants import THREAD_PAYLOAD, THREAD_CHECK_SOFTWARE, \
            THREAD_SOFTWARE_WATCHER, PAYLOAD_TYPE_DNF

from simpleline.render.containers import ListColumnContainer
from simpleline.render.screen import InputState
from simpleline.render.screen_handler import ScreenHandler
from simpleline.render.widgets import TextWidget, CheckboxWidget

from pyanaconda.anaconda_loggers import get_module_logger
log = get_module_logger(__name__)

__all__ = ["KernelSpoke"]


class KernelSpoke(NormalTUISpoke):
    """ Spoke used to read new value of text to represent source repo.

       .. inheritance-diagram:: SoftwareSpoke
          :parts: 3
    """
    help_id = "KernelSelectionSpoke"
    category = SoftwareCategory

    def __init__(self, data, storage, payload):
        super().__init__(data, storage, payload)
        self.title = N_("Kernel selection")
        self._container = None
        self.errors = []

        self._orig_kernel = None
        self._kickstarted = flags.automatedInstall and self.data.packages.seen
        repo_path = "/run/install/repo/"
        repo_list = os.listdir(repo_path)
        self._kernel_repos = []
        kernel_repo_name = "kernel"
        for repo in repo_list:
            if kernel_repo_name in repo:
                self._kernel_repos.append(repo)

        payloadMgr.add_listener(PayloadState.STARTED, self._payload_start)
        payloadMgr.add_listener(PayloadState.FINISHED, self._payload_finished)
        payloadMgr.add_listener(PayloadState.ERROR, self._payload_error)

    def _payload_start(self):
        # Source is changing, invalidate the software selection and clear the
        # errors
        self.errors = []
        self.current_kernel = None

    def _payload_finished(self):
        msgs = self.payload.detectMultiKernel()
        if not msgs:
            # use latest kernel as default, normally it's anck
            self.current_kernel = list(self.available_kernels.keys())[-1]

    def _payload_error(self):
        self.errors = [payloadMgr.error]

    def initialize(self):
        self.initialize_start()
        super().initialize()
        threadMgr.add(AnacondaThread(name="AnaKernelWatcher", target=self._initialize))

    def _initialize(self):
        threadMgr.wait(THREAD_PAYLOAD)

        self._apply()

        threadMgr.wait("AnaCheckKernel")
        self.initialize_done()

    @property
    def changed(self):
        if self.current_kernel == self._orig_kernel:
            return False
        else:
            return True

    @property
    def showable(self):
        return self.payload.type == PAYLOAD_TYPE_DNF

    @property
    def status(self):
        if not self.payload.base_repo:
            return _("Installation source not set up")

        if not self.ready:
            return _("Processing...")

        if self.errors:
            return _("Error checking kernel selection")

        if not self._kickstarted:
            if not self.current_kernel:
                return _("Please confirm kernel selection")

        return self.current_kernel

    @property
    def completed(self):
        processing_done = self.ready and not self.errors
        return processing_done and self.payload.base_repo

    def refresh(self, args=None):
        """ Refresh screen. """
        NormalTUISpoke.refresh(self, args)

        threadMgr.wait(THREAD_PAYLOAD)
        self._container = None

        if not self.payload.base_repo:
            message = TextWidget(_("Installation source needs to be set up first."))
            self.window.add_with_separator(message)
            return

        threadMgr.wait("AnaCheckKernel")
        self._container = ListColumnContainer(2, columns_width=38, spacing=2)

        msg = self._refresh_kernels()

        self.window.add_with_separator(TextWidget(msg))
        self.window.add_with_separator(self._container)

    def _refresh_kernels(self):
        for kernel in self._kernel_repos:
            selected = (self.current_kernel == kernel)
            if kernel == 'kernel419':
                kernel_desc = _("RTM4")
                kernel_info = _("RTM4 NFSCNS Generic")
            elif kernel == 'kernel510':
                kernel_desc = _("RTM4")
                kernel_info = _("RTM4 NFSCNS 5.10")
            else:
                kernel_desc = ''
                kernel_info = ''
            widget = CheckboxWidget(title=kernel, completed=selected)
            self._container.add(widget, callback=self._set_kernel_callback, data=kernel)
        return _("Kernel List")

    def _set_kernel_callback(self, data):
        self.current_kernel = data

    def input(self, args, key):
        if self._container is not None and self._container.process_user_input(key):
            self.redraw()
        else:
            # TRANSLATORS: 'c' to continue
            if key.lower() == C_('TUI|Spoke Navigation', 'c'):

                if self.current_kernel is None:
                    self.close()
                else:
                    self.apply()
                    self.close()
            else:
                return super().input(args, key)

        return InputState.PROCESSED

    @property
    def ready(self):
        """ If we're ready to move on. """
        return (not threadMgr.get(THREAD_PAYLOAD) and
                not threadMgr.get("AnaCheckKernel") and
                not threadMgr.get("AnaKernelWatcher"))

    @property
    def available_kernels(self):
        return self.payload.available_kernels

    @property
    def current_kernel(self):
        return self.payload.current_kernel

    @current_kernel.setter
    def current_kernel(self, value):
        self.payload.current_kernel = value

    def apply(self):
        self._apply()

    def _apply(self):
        threadMgr.add(AnacondaThread(name="AnaCheckKernel", target=self.checkKernelSelection))


    def checkKernelSelection(self):
        if self._kickstarted:
            kernel_pattern=re.compile(r"kernel-[4,5].\d+.\d+")
            for package in self.payload.data.packages.packageList:
                if kernel_pattern.search(package):
                    self.current_kernel = package.split('-')[1]

        # we do this only kernel changed
        if self.changed:
            if os.path.exists("/tmp/.anaconda_payload_finished"):
                os.remove("/tmp/.anaconda_payload_finished")
            self._repos_backup = copy.copy(self.data.repo.dataList())
            for r in self._repos_backup:
                if r.name in self._kernel_repos:
                    self.payload.data.repo.dataList().remove(r)
                    if r.name == self.current_kernel:
                        r.enabled = True
                    else:
                        r.enabled = False
                    self.payload.data.repo.dataList().append(r)
            payloadMgr.restart_thread(self.payload, checkmount=False)
            self._orig_kernel = self.current_kernel
            threadMgr.wait(THREAD_PAYLOAD)
            threadMgr.wait(THREAD_CHECK_SOFTWARE)
        while not os.path.exists("/tmp/.anaconda_payload_finished"):
            log.debug("check_software_selection: wait payload finished...")
            sleep(3)
